@@ -39,7 +39,6 @@ from zun.container import driver
 from zun.image import driver as img_driver
 from zun.network import network as zun_network
 from zun import objects
-from zun.volume import driver as vol_driver
 
 
 CONF = zun.conf.CONF
@@ -105,7 +104,8 @@ def wrap_docker_error(function):
     return decorated_function
 
 
-class DockerDriver(driver.ContainerDriver):
+class DockerDriver(driver.BaseDriver, driver.ContainerDriver,
+                   driver.CapsuleDriver):
     """Implementation of container drivers for Docker."""
 
     # TODO(hongbin): define a list of capabilities of this driver.
@@ -119,10 +119,6 @@ class DockerDriver(driver.ContainerDriver):
         for driver_name in CONF.image_driver_list:
             driver = img_driver.load_image_driver(driver_name)
             self.image_drivers[driver_name] = driver
-        self.volume_drivers = {}
-        for driver_name in CONF.volume.driver_list:
-            driver = vol_driver.driver(driver_name)
-            self.volume_drivers[driver_name] = driver
 
     def _get_host_storage_info(self):
         host_info = self.get_host_info()
@@ -269,6 +265,7 @@ class DockerDriver(driver.ContainerDriver):
                 'tty': container.tty,
                 'stdin_open': container.interactive,
                 'hostname': container.hostname,
+                'entrypoint': container.entrypoint,
             }
 
             if not self._is_runtime_supported():
@@ -383,6 +380,8 @@ class DockerDriver(driver.ContainerDriver):
                                    requested_networks, host_config,
                                    container_kwargs, docker):
         network_api = zun_network.api(context=context, docker_api=docker)
+        neutron_api = network_api.neutron_api
+
         # Process the first requested network at create time. The rest
         # will be processed after create.
         requested_network = requested_networks.pop()
@@ -390,9 +389,12 @@ class DockerDriver(driver.ContainerDriver):
             context, requested_network['network'])
         security_group_ids = utils.get_security_group_ids(
             context, container.security_groups)
-        addresses, port = network_api.create_or_update_port(
-            container, docker_net_name, requested_network, security_group_ids,
-            set_binding_host=True)
+        docker_network = network_api.inspect_network(docker_net_name)
+        device_owner = network_api.get_device_owner()
+        neutron_net_id = docker_network['Options']['neutron.net.uuid']
+        addresses, port = neutron_api.create_or_update_port(
+            container, neutron_net_id, requested_network, device_owner,
+            security_group_ids, set_binding_host=True)
         container.addresses = {requested_network['network']: addresses}
 
         ipv4_address = None
@@ -1030,35 +1032,6 @@ class DockerDriver(driver.ContainerDriver):
             value = six.text_type(value)
         return value.encode('utf-8')
 
-    def _get_volume_driver(self, volume_mapping):
-        driver_name = volume_mapping.volume_provider
-        driver = self.volume_drivers.get(driver_name)
-        if not driver:
-            msg = _("The volume provider '%s' is not supported") % driver_name
-            raise exception.ZunException(msg)
-
-        return driver
-
-    def attach_volume(self, context, volume_mapping):
-        volume_driver = self._get_volume_driver(volume_mapping)
-        volume_driver.attach(context, volume_mapping)
-
-    def detach_volume(self, context, volume_mapping):
-        volume_driver = self._get_volume_driver(volume_mapping)
-        volume_driver.detach(context, volume_mapping)
-
-    def delete_volume(self, context, volume_mapping):
-        volume_driver = self._get_volume_driver(volume_mapping)
-        volume_driver.delete(context, volume_mapping)
-
-    def is_volume_available(self, context, volume_mapping):
-        volume_driver = self._get_volume_driver(volume_mapping)
-        return volume_driver.is_volume_available(context, volume_mapping)
-
-    def is_volume_deleted(self, context, volume_mapping):
-        volume_driver = self._get_volume_driver(volume_mapping)
-        return volume_driver.is_volume_deleted(context, volume_mapping)
-
     def _get_or_create_docker_network(self, context, network_api,
                                       neutron_net_id):
         docker_net_name = self._get_docker_network_name(context,
@@ -1103,7 +1076,6 @@ class DockerDriver(driver.ContainerDriver):
             else:
                 runtimes = ['runc']
             docker_root_dir = info['DockerRootDir']
-            enable_cpu_pinning = CONF.compute.enable_cpu_pinning
 
             return {'total_containers': total,
                     'running_containers': running,
@@ -1116,8 +1088,7 @@ class DockerDriver(driver.ContainerDriver):
                     'kernel_version': kernel_version,
                     'labels': labels,
                     'runtimes': runtimes,
-                    'docker_root_dir': docker_root_dir,
-                    'enable_cpu_pinning': enable_cpu_pinning}
+                    'docker_root_dir': docker_root_dir}
 
     def get_total_disk_for_container(self):
         try:
@@ -1153,6 +1124,24 @@ class DockerDriver(driver.ContainerDriver):
     def get_available_nodes(self):
         return [self._host.get_hostname()]
 
+    def get_available_resources(self):
+        data = super(DockerDriver, self).get_available_resources()
+
+        info = self.get_host_info()
+        data['total_containers'] = info['total_containers']
+        data['running_containers'] = info['running_containers']
+        data['paused_containers'] = info['paused_containers']
+        data['stopped_containers'] = info['stopped_containers']
+        data['cpus'] = info['cpus']
+        data['architecture'] = info['architecture']
+        data['os_type'] = info['os_type']
+        data['os'] = info['os']
+        data['kernel_version'] = info['kernel_version']
+        data['labels'] = info['labels']
+        data['runtimes'] = info['runtimes']
+
+        return data
+
     @wrap_docker_error
     def network_detach(self, context, container, network):
         with docker_utils.docker_client() as docker:
@@ -1227,6 +1216,7 @@ class DockerDriver(driver.ContainerDriver):
                                               requested_networks,
                                               requested_volumes)
             self._wait_for_init_container(context, container)
+            container.save(context)
         for container in capsule.containers:
             self._create_container_in_capsule(context, capsule, container,
                                               requested_networks,
@@ -1265,6 +1255,7 @@ class DockerDriver(driver.ContainerDriver):
                 'labels': container.labels,
                 'tty': container.tty,
                 'stdin_open': container.interactive,
+                'entrypoint': container.entrypoint,
             }
 
             host_config = {}
@@ -1338,9 +1329,16 @@ class DockerDriver(driver.ContainerDriver):
         r.call(check_init_container_stopped)
 
     def delete_capsule(self, context, capsule, force):
-        for container in capsule.containers:
+        merged_containers = capsule.containers + capsule.init_containers
+        for container in merged_containers:
             self._delete_container_in_capsule(context, capsule, container,
                                               force)
+        with docker_utils.docker_client() as docker:
+            try:
+                docker.stop(capsule.container_id)
+            except errors.APIError as api_error:
+                if is_not_found(api_error):
+                    pass
         self.delete(context, capsule, force)
 
     def _delete_container_in_capsule(self, context, capsule, container, force):
